package com.jiuqi.util;

import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.UnsupportedEncodingException;
import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStream;

public abstract class Stream
{
    private boolean encode;
    private String charset;
    private static final int BUFF_SIZE = 1024;
    public static final int SEEK_ORIGIN_START = 0;
    public static final int SEEK_ORIGIN_CURRENT = 1;
    public static final int SEEK_ORIGIN_END = 2;
    
    public Stream() {
        this.encode = true;
        this.charset = "GBK";
    }
    
    public void setUseEncode(final boolean useEncode) {
        this.encode = useEncode;
    }
    
    public boolean getUseEncode() {
        return this.encode;
    }
    
    public void setCharset(final String encodeCharset) {
        if (encodeCharset != null && encodeCharset.length() != 0) {
            this.charset = encodeCharset;
        }
    }
    
    public String getCharset() {
        return this.charset;
    }
    
    public abstract void setSize(final long p0) throws StreamException;
    
    public abstract int read(final byte[] p0, final int p1, final int p2) throws StreamException;
    
    public abstract byte read() throws StreamException;
    
    public abstract int write(final byte[] p0, final int p1, final int p2) throws StreamException;
    
    public abstract void write(final byte p0) throws StreamException;
    
    public abstract long seek(final long p0, final int p1) throws StreamException;
    
    public void readBuffer(final byte[] buffer, final int pos, final int count) throws StreamException {
        if (count != 0 && this.read(buffer, pos, count) != count) {
            throw new StreamException("Stream read error");
        }
    }
    
    public void writeBuffer(final byte[] buffer, final int pos, final int count) throws StreamException {
        if (count != 0 && this.write(buffer, pos, count) != count) {
            throw new StreamException("Stream write error");
        }
    }
    
    public long copyFrom(final Stream source, long count) throws StreamException {
        if (source == null) {
            return 0L;
        }
        if (count == 0L) {
            source.setPosition(0L);
            count = source.getSize();
        }
        int bufSize;
        if (count > 1024L) {
            bufSize = 1024;
        }
        else {
            bufSize = (int)count;
        }
        final byte[] buffer = new byte[1024];
        while (count != 0L) {
            int n;
            if (count > bufSize) {
                n = bufSize;
            }
            else {
                n = (int)count;
            }
            source.readBuffer(buffer, 0, n);
            this.writeBuffer(buffer, 0, n);
            count -= n;
        }
        return 0L;
    }
    
    public long copyFrom(final InputStream stream, long size) throws StreamException, IOException {
        if (stream == null) {
            return 0L;
        }
        long readcount = 0L;
        if (size == 0L) {
            final byte[] buf = new byte[1024];
            int b = 0;
            do {
                b = stream.read(buf);
                if (b > 0) {
                    this.write(buf, 0, b);
                    readcount += b;
                }
            } while (b > 0);
            while (b > 0) {
                b = stream.read(buf);
                if (b > 0) {
                    this.write(buf, 0, b);
                }
            }
        }
        else {
            final byte[] buffer = new byte[1024];
            int readSize;
            do {
                final int blockSize = (size > 1024L) ? 1024 : ((int)size);
                readSize = stream.read(buffer, 0, blockSize);
                this.write(buffer, 0, readSize);
                readcount += readSize;
                size -= readSize;
            } while (readSize > 0 && size > 0L);
        }
        return readcount;
    }
    
    public long copyTo(final OutputStream stream, long size) throws StreamException, IOException {
        long writecount = 0L;
        final byte[] buffer = new byte[1024];
        for (int writesize = this.read(buffer, 0, 1024); writesize > 0 && size > 0L; writesize = this.read(buffer, 0, 1024), writecount += writesize, size -= writesize) {
            stream.write(buffer, 0, writesize);
        }
        return writecount;
    }
    
    public byte[] getBytes() throws StreamException {
        final long oldPos = this.getPosition();
        try {
            final byte[] rt = new byte[(int)this.getSize()];
            this.setPosition(0L);
            this.read(rt, 0, rt.length);
            return rt;
        }
        finally {
            this.setPosition(oldPos);
        }
    }
    
    public long getSize() throws StreamException {
        final long pos = this.seek(0L, 1);
        final long r = this.seek(0L, 2);
        this.seek(pos, 0);
        return r;
    }
    
    public long getPosition() throws StreamException {
        return this.seek(0L, 1);
    }
    
    public void setPosition(final long pos) throws StreamException {
        this.seek(pos, 0);
    }
    
    public short readShort() throws StreamException {
        final int b1 = this.read() & 0xFF;
        final int b2 = this.read() & 0xFF;
        return (short)((b2 << 8) + (b1 << 0));
    }
    
    public int readWord() throws StreamException {
        final int b1 = this.read() & 0xFF;
        final int b2 = this.read() & 0xFF;
        return (b2 << 8) + (b1 << 0);
    }
    
    public int readInt() throws StreamException {
        final int b1 = this.read() & 0xFF;
        final int b2 = this.read() & 0xFF;
        final int b3 = this.read() & 0xFF;
        final int b4 = this.read() & 0xFF;
        return (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }
    
    public float readFloat() throws StreamException {
        return Float.intBitsToFloat(this.readInt());
    }
    
    public double readDouble() throws StreamException {
        return Double.longBitsToDouble(this.readLong());
    }
    
    public String readString(final int length) throws StreamException {
        final byte[] str = new byte[length];
        this.read(str, 0, length);
        return this.decodeString(str);
    }
    
    public String decodeString(final byte[] data) {
        if (this.encode) {
            try {
                return new String(data, this.charset);
            }
            catch (final UnsupportedEncodingException ex) {
                return new String(data);
            }
        }
        return new String(data);
    }
    
    public String readStringBySize() throws StreamException {
        final int len = this.readInt();
        if (len == 0) {
            return null;
        }
        return this.readString(len);
    }
    
    public boolean readBool() throws StreamException {
        return this.read() > 0;
    }
    
    public long readLong() throws StreamException {
        final long b1 = this.read() & 0xFF;
        final long b2 = this.read() & 0xFF;
        final long b3 = this.read() & 0xFF;
        final long b4 = this.read() & 0xFF;
        final long b5 = this.read() & 0xFF;
        final long b6 = this.read() & 0xFF;
        final long b7 = this.read() & 0xFF;
        final long b8 = this.read();
        return (b8 << 56) + (b7 << 48) + (b6 << 40) + (b5 << 32) + (b4 << 24) + (b3 << 16) + (b2 << 8) + (b1 << 0);
    }
    
    public void writeShort(final short value) throws StreamException {
        this.write((byte)(value >>> 0 & 0xFF));
        this.write((byte)(value >>> 8 & 0xFF));
    }
    
    public void writeWord(final int value) throws StreamException {
        this.write((byte)(value >>> 0 & 0xFF));
        this.write((byte)(value >>> 8 & 0xFF));
    }
    
    public void writeInt(final int value) throws StreamException {
        this.write((byte)(value >>> 0 & 0xFF));
        this.write((byte)(value >>> 8 & 0xFF));
        this.write((byte)(value >>> 16 & 0xFF));
        this.write((byte)(value >>> 24 & 0xFF));
    }
    
    public void writeLong(final long value) throws StreamException {
        this.write((byte)(value >>> 0 & 0xFFL));
        this.write((byte)(value >>> 8 & 0xFFL));
        this.write((byte)(value >>> 16 & 0xFFL));
        this.write((byte)(value >>> 24 & 0xFFL));
        this.write((byte)(value >>> 32 & 0xFFL));
        this.write((byte)(value >>> 40 & 0xFFL));
        this.write((byte)(value >>> 48 & 0xFFL));
        this.write((byte)(value >>> 56 & 0xFFL));
    }
    
    public void writeFloat(final float value) throws StreamException {
        this.writeInt(Float.floatToIntBits(value));
    }
    
    public void writeDouble(final double value) throws StreamException {
        this.writeLong(Double.doubleToLongBits(value));
    }
    
    public void writeBool(final boolean value) throws StreamException {
        final byte b = (byte)(value ? 1 : 0);
        this.write(b);
    }
    
    public void writeString(final String value) throws StreamException {
        final byte[] b = this.encodeString(value);
        this.write(b, 0, b.length);
    }
    
    public byte[] encodeString(final String value) {
        if (value == null) {
            return null;
        }
        if (this.encode) {
            try {
                return value.getBytes(this.charset);
            }
            catch (final UnsupportedEncodingException ex) {
                return value.getBytes();
            }
        }
        return value.getBytes();
    }
    
    public void writeStringWithSize(final String value) throws StreamException {
        if (value == null) {
            this.writeInt(0);
            return;
        }
        final byte[] b = this.encodeString(value);
        this.writeInt(b.length);
        this.write(b, 0, b.length);
    }
    
    public void loadFromFile(final String fileName) throws StreamException, IOException {
        final InputStream source = new FileInputStream(fileName);
        try {
            this.loadFromStream(source);
        }
        finally {
            source.close();
        }
        source.close();
    }
    
    public void loadFromStream(final InputStream inStream) throws StreamException, IOException {
        this.copyFrom(inStream, 0L);
    }
    
    public void saveToFile(final String fileName) throws StreamException, IOException {
        final OutputStream dest = new FileOutputStream(fileName);
        try {
            this.saveToStream(dest);
        }
        finally {
            dest.close();
        }
        dest.close();
    }
    
    public void saveToStream(final OutputStream outStream) throws StreamException, IOException {
        this.setPosition(0L);
        this.copyTo(outStream, this.getSize());
    }
}
